﻿Imports Infragistics.Win
Imports Infragistics.Win.UltraWinTree
Public Class Form1

    Private Const STYLE_HEADER As String = "MyHeader"
    Private Const STYLE_BOLD As String = "MyBold"
    Private Const STYLE_GRAY As String = "MyGray"
    Private Const STYLE_RED As String = "MyRed"
    Private Const STYLE_BLUE As String = "MyBlue"

    Private Enum PlanCareTypes As Short
        Order = 1  ' For type of problem in problem list
        Active = 2
        Inactive = 3
    End Enum


    Private strHeaders() As String = {"Clinical Orders", "Active Nursing Diagnoses", "Inactive Nursing Diagnose"}
    Private myPlanCareNodeDupeIndex As Integer
    Private myMenuActive As ContextMenu     ' For POC Active
    Private myMenuRNDx As ContextMenu       ' For POC Dx
    Private myMenuRNRx As ContextMenu       ' For POC Rx
    Private myMenuRNOx As ContextMenu       ' For POC Ox
    Private myPOCEventData As CTreeNodeTag = Nothing
    Private myMouse As System.Windows.Forms.MouseEventArgs = Nothing

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        CreateContextMenus()
        InitPlanOfCare()
    End Sub

    Private Sub AddHeaders2PlanOfCare()
        Dim Row As Integer
        Dim keys() As String = {"O", "A", "I"}
        Dim ProblemTypes() As PlanCareTypes = {PlanCareTypes.Order, PlanCareTypes.Active, PlanCareTypes.Inactive}

        Dim utNode As UltraTreeNode
        For Row = 0 To 2
            utNode = utPlanCare.Nodes.Add(keys(Row), strHeaders(Row))
            utNode.Override.NodeAppearance = utPlanCare.Appearances(STYLE_HEADER)
            utNode.Tag = ProblemTypes(Row)  ' Store the problem type in the tag
            If (Row = 0) Then
                ' Assign the 'orders' column set to the orders node node's collection
                utNode.Nodes.Override.ColumnSet = utPlanCare.ColumnSettings.ColumnSets("order")
            End If
        Next
    End Sub

    ''' <summary>
    ''' Procedure adds a clinical order to the Plan of Care Orders node
    ''' </summary>
    ''' <param name="oData">The data item to add.</param>
    ''' <remarks></remarks>
    Private Sub AddPlanOfCareOrder(ByVal oData As CTreeNodeTag)
        Try
            Dim oNode As UltraTreeNode = utPlanCare.Nodes(0).Nodes.Add()
            With oNode
                .Cells(0).Value = FormatDateTime(Mdate2Date(oData.Etime), vbShortDate)
                .Cells(1).Value = oData.Caption
                .Tag = oData
            End With
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    Private Sub CreateContextMenus()
        ' This creates the context menus for the Plan of Care
        Dim mItem As MenuItem
        Dim Index As Short
        Dim strCaps() As String
        Try
            ' POC Active
            myMenuActive = New ContextMenu
            strCaps = Split("Ignore Action|-|Refresh POC|Flow Sheet POC|Flow Sheet Active Diagnoses|Quick Print|Add New Nursing Diagnosis", "|")
            For Index = 0 To UBound(strCaps)
                If (strCaps(Index).Equals("-")) Then
                    mItem = New MenuItem(strCaps(Index))
                Else
                    mItem = New MenuItem(strCaps(Index), New System.EventHandler(AddressOf Me.MenuActive))
                End If
                myMenuActive.MenuItems.Add(mItem)
            Next
            ' POC Dx
            myMenuRNDx = New ContextMenu
            strCaps = Split("Ignore Action|-|Flow Sheet This Diagnosis|Nurse Prompt|Resolve This Diagnosis|Quick Interventions", "|")
            For Index = 0 To UBound(strCaps)
                If (strCaps(Index).Equals("-")) Then
                    mItem = New MenuItem(strCaps(Index))
                Else
                    mItem = New MenuItem(strCaps(Index), New System.EventHandler(AddressOf Me.MenuRNDx))
                End If
                myMenuRNDx.MenuItems.Add(mItem)
            Next
            ' POC Rx
            myMenuRNRx = New ContextMenu
            strCaps = Split("Ignore Action|-|Nurse Prompt|Quick Nursing Orders", "|")
            For Index = 0 To UBound(strCaps)
                If (strCaps(Index).Equals("-")) Then
                    mItem = New MenuItem(strCaps(Index))
                Else
                    mItem = New MenuItem(strCaps(Index), New System.EventHandler(AddressOf Me.MenuRNRx))
                End If
                myMenuRNRx.MenuItems.Add(mItem)
            Next
            ' POC Ox
            myMenuRNOx = New ContextMenu
            strCaps = Split("Ignore Action|-|Quick Assess|Quick Perform|Quick Teach|Quick Manage", "|")
            For Index = 0 To UBound(strCaps)
                If (strCaps(Index).Equals("-")) Then
                    mItem = New MenuItem(strCaps(Index))
                Else
                    mItem = New MenuItem(strCaps(Index), New System.EventHandler(AddressOf Me.menuRNOx))
                End If
                myMenuRNOx.MenuItems.Add(mItem)
            Next
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    Private Sub MenuActive(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim mItem As MenuItem = DirectCast(sender, MenuItem)
        Dim Index As Integer = myMenuActive.MenuItems.IndexOf(mItem)
        'strCaps = Split("Ignore Action|-|Refresh POC|Flow Sheet POC|Flow Sheet Active Diagnoses|Quick Print|Add New Nursing Diagnosis", "|")
        Try
            Select Case Index
                Case 2
                    ' Refresh POC
                Case 3
                    ' Flow Sheet POC
                Case 4
                    ' Flow Sheet Active Diagnoses
                Case 5
                    ' Quick Print
                Case 6
                    ' Add New Nursing Diagnosis
                Case Else
            End Select
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    Private Sub MenuRNDx(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim mItem As MenuItem = DirectCast(sender, MenuItem)
        Dim Index As Integer = myMenuRNDx.MenuItems.IndexOf(mItem)
        'Dim strProblems As String
        'strCaps = Split("Ignore Action|-|Flow Sheet This Diagnosis|Nurse Prompt|Resolve This Diagnosis|Quick Interventions", "|")
        Select Case Index
            Case 2
                ' Flow Sheet This Diagnosis
            Case 3
                ' Nurse Prompt
            Case 4
                ' Resolve This Diagnosis
            Case 5
                ' Quick Interventions
            Case Else
        End Select
    End Sub

    Private Sub MenuRNRx(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim mItem As MenuItem = DirectCast(sender, MenuItem)
        Dim Index As Integer = myMenuRNRx.MenuItems.IndexOf(mItem)
        'Dim strProblems As String
        'strCaps = Split("Ignore Action|-|Nurse Prompt|Quick Nursing Orders", "|")
        Select Case Index
            Case Else
        End Select
    End Sub

    Private Sub MenuRNOx(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim mItem As MenuItem = DirectCast(sender, MenuItem)
        Dim Index As Integer = myMenuRNOx.MenuItems.IndexOf(mItem)
        'Dim strProblems As String
        'strCaps = Split("Ignore Action|-|Quick Assess|Quick Perform|Quick Teach|Quick Manage", "|")
        Select Case Index
            Case Else
        End Select
    End Sub

    Private Sub InitPlanOfCare()
        Static DidInit As Boolean = False
        If (DidInit) Then Exit Sub
        DidInit = True
        ' Procedure to set up the properties of the ProblemList UltraWinTree
        Dim ColSet As UltraTreeColumnSet
        Dim Style As Infragistics.Win.Appearance
        Try
            With utPlanCare
                If (.Appearances.Count = 0) Then
                    Style = .Appearances.Add(STYLE_HEADER)
                    Style.ForeColor = Color.Blue
                    Style.FontData.Bold = DefaultableBoolean.True
                    Style = .Appearances.Add(STYLE_BOLD)
                    Style.FontData.Bold = DefaultableBoolean.True
                    Style = .Appearances.Add(STYLE_RED)
                    Style.ForeColor = Color.Red
                    Style = .Appearances.Add(STYLE_BLUE)
                    Style.ForeColor = Color.Blue
                End If
                .HideSelection = False
                .Indent = 8
                '.Override.Multiline = DefaultableBoolean.True
                .Override.CellClickAction = UltraWinTree.CellClickAction.SelectNodeOnly
                .Override.SelectionType = UltraWinTree.SelectType.SingleAutoDrag
                .Override.TipStyleNode = TipStyleNode.Hide
                '.ColumnSettings.AutoFitColumns = AutoFitColumns.ResizeAllColumns
                .ViewStyle = UltraWinTree.ViewStyle.FreeForm
                ColSet = .ColumnSettings.ColumnSets("rnox")
                ColSet.TipStyleCell = TipStyleCell.Show
                ColSet.CellWrapText = DefaultableBoolean.True
                ColSet = .ColumnSettings.ColumnSets("order")
                ColSet.TipStyleCell = TipStyleCell.Show
            End With
            AddHeaders2PlanOfCare()
            ResizePlanOfCare()
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    Private Function Mdate2Date(ByVal sDate As String) As Date
        ' Convert date in yyyymmddhhmm form to date
        Dim yr As Short, mon As Short, day As Short
        Dim hr, min, sec As Short
        Dim slen As Short

        Try
            slen = Len(sDate)
            If (slen >= 4) Then yr = Val(Mid(sDate, 1, 4))
            If (slen >= 6) Then mon = Val(Mid(sDate, 5, 2))
            If (slen >= 8) Then day = Val(Mid(sDate, 7, 2))
            If (slen >= 10) Then
                hr = Val(Mid(sDate, 9, 2))
                If (slen >= 12) Then
                    min = Val(Mid(sDate, 11, 2))
                    If (slen >= 14) Then
                        sec = Val(Mid(sDate, 13, 2))
                    Else
                        sec = 0
                    End If
                    Mdate2Date = CDate(mon & "/" & day & "/" & yr & " " & hr & ":" & min & ":" & sec)
                Else
                End If
            Else
                Mdate2Date = CDate(mon & "/" & day & "/" & yr)
            End If
        Catch ex As System.Exception
            MsgBox("Mdate2Date: " & ex.Message, MsgBoxStyle.Exclamation)
        End Try
    End Function

    ''' <summary>
    ''' Procedure adds a RNDx (nursing diagnoses) to the Plan of Care 
    ''' </summary>
    ''' <param name="oData"></param>
    ''' <remarks>Each addition must be a unique Nodekey.</remarks>
    Private Sub AddPlanOfCareRNDx(ByVal oData As CTreeNodeTag)
        Dim oNode As UltraTreeNode
        Dim Index As Integer
        Dim strPrefix As String = String.Empty
        Select Case oData.Prefix
            Case "O1"
                Index = 2
                strPrefix = "Outcome: Improved"
            Case "O2"
                Index = 2
                strPrefix = "Outcome: Stabilized"
            Case "O3"
                Index = 2
                strPrefix = "Outcome: Suported"
            Case "G1"
                Index = 1
                strPrefix = "Goal: Improve"
            Case "G2"
                Index = 1
                strPrefix = "Goal: Stabilize"
            Case "G3"
                Index = 1
                strPrefix = "Goal: Support"
            Case Else
                Index = 1
        End Select
        Try
            With utPlanCare.Nodes(Index)
                Dim strCaption As String = oData.Caption & " (" & oData.CCC & ")"
                If (Not strPrefix.Equals(String.Empty)) Then
                    strCaption = strCaption & " " & strPrefix
                End If
                oNode = .Nodes.Add(oData.NodeKey, strCaption)
                oNode.Tag = oData
                oNode.Override.BorderStyleNode = UIElementBorderStyle.TwoColor
                'oNode.Override.NodeAppearance.ForeColor = Color.Blue
                oNode.Override.NodeAppearance = utPlanCare.Appearances(STYLE_BLUE)
            End With
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    ''' <summary>
    ''' Procedure adds an RNOx (nursing intervention) to the Plan of Care 
    ''' </summary>
    ''' <param name="oData"></param>
    ''' <remarks>Each addition must be a unique Nodekey.</remarks>
    Private Sub AddPlanOfCareRNOx(ByVal oData As CTreeNodeTag)
        If (oData.CCC.Length < 7) Then Exit Sub
        Dim oNode, ndDx, ndRx As UltraTreeNode
        Dim Group As Integer
        Dim oTag As CTreeNodeTag
        Dim strPrefix As String = String.Empty
        Try
            ' Find an RNRx with a matching base CCC code
            For Group = 1 To 2
                For Each ndDx In utPlanCare.Nodes(Group).Nodes
                    For Each ndRx In ndDx.Nodes
                        oTag = ndRx.Tag
                        If (oData.CCC.Substring(0, 5).Equals(oTag.CCC)) Then
                            Dim iCell As Integer
                            If (oData.CCC.EndsWith("1")) Then
                                iCell = 0
                            ElseIf (oData.CCC.EndsWith("2")) Then
                                iCell = 1
                            ElseIf (oData.CCC.EndsWith("3")) Then
                                iCell = 2
                            Else
                                iCell = 3
                            End If

                            ' Get next row needed for this data cell
                            oNode = Nothing
                            If (Not ndRx.HasNodes) Then
                                oNode = ndRx.Nodes.Add()
                                'oNode.Tag = ndRx.Tag
                            Else
                                Dim bolHaveRNOxNode As Boolean = False
                                For Each oNode In ndRx.Nodes
                                    If (oNode.Cells.Count <= iCell OrElse oNode.Cells(iCell).Text = String.Empty) Then
                                        bolHaveRNOxNode = True
                                        Exit For
                                    End If
                                Next
                                If (Not bolHaveRNOxNode) Then
                                    oNode = ndRx.Nodes.Add()
                                    'oNode.Tag = ndRx.Tag
                                End If
                            End If
                            Dim strCaption As String = oData.Caption
                            If (oData.SIG.Length > 0) Then
                                strCaption = strCaption & oData.SIG
                            End If
                            With oNode
                                .Cells(iCell).Tag = oData
                                .Cells(iCell).Value = strCaption
                            End With
                        End If
                    Next
                Next
            Next
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    ''' <summary>
    ''' Procedure adds an RNRx (nursing intervention) to the Plan of Care 
    ''' </summary>
    ''' <param name="oData"></param>
    ''' <remarks>Each addition must be a unique Nodekey.</remarks>
    Private Sub AddPlanOfCareRNRx(ByVal oData As CTreeNodeTag)
        Dim oNode, oParent As UltraTreeNode
        Dim Group As Integer
        Dim oDx As CTreeNodeTag
        Try
            Dim strCaption As String = oData.Caption & " (" & oData.CCC & ")"
            If (oData.Status <> String.Empty) Then
                strCaption = strCaption & " " & oData.Status
            End If
            ' Find an RNDx that this is linked to; look in active and inactive RNDx nodes
            For Group = 1 To 2
                For Each oParent In utPlanCare.Nodes(Group).Nodes
                    Dim bolFound As Boolean = False
                    oDx = oParent.Tag
                    For Each oItem As CIndexItem In oDx.RNRxList
                        If (oData.NodeKey.Length >= oItem.NodeKey.Length AndAlso oData.NodeKey.Substring(0, oItem.NodeKey.Length).Equals(oItem.NodeKey)) Then
                            bolFound = True
                            Exit For
                        End If
                    Next
                    If (bolFound) Then
                        Dim strKey As String
                        oNode = utPlanCare.GetNodeByKey(oData.NodeKey)
                        If (oNode Is Nothing) Then
                            strKey = oData.NodeKey
                        Else
                            myPlanCareNodeDupeIndex += 1
                            strKey = oData.NodeKey & Format(myPlanCareNodeDupeIndex, "00")
                        End If
                        oNode = oParent.Nodes.Add(strKey, strCaption)
                        oNode.Tag = oData
                        ' Assign the 'rnox' column set to the orders node node's collection
                        oNode.Nodes.Override.ColumnSet = utPlanCare.ColumnSettings.ColumnSets("rnox")
                    End If
                Next
            Next
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub

    Private Sub BuildPlanOfCare(ByVal arData As ArrayList)
        Dim objNode, objSibling, objBNode As UltraTreeNode
        'Dim Status As Integer
        Dim oData As CTreeNodeTag
        Dim arOrders As New ArrayList
        Dim arDx As New ArrayList
        Dim arRx As New ArrayList
        Dim arOx As New ArrayList
        myPlanCareNodeDupeIndex = 0
        'Dim bFileOpen As Boolean
        Try
            utPlanCare.BeginUpdate()
            ClearPlanOfCare()

            If (arData.Count > 0) Then
                For Each oData In arData
                    Select Case oData.RNType
                        Case CTreeNodeTag.RnTypes.None
                            arOrders.Add(oData)
                        Case CTreeNodeTag.RnTypes.Dx
                            arDx.Add(oData)
                        Case CTreeNodeTag.RnTypes.Rx
                            arRx.Add(oData)
                        Case CTreeNodeTag.RnTypes.Ox
                            arOx.Add(oData)
                    End Select
                Next
            End If
            'Dim FileName As String = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) & "\pocdump.txt"
                'FileOpen(1, FileName, OpenMode.Output, OpenAccess.Write, OpenShare.Shared)
                'bFileOpen = True

                Dim myComparer As New CCompareTagMedcinId
                If (arOrders.Count > 0) Then
                    Dim LastData As CTreeNodeTag = Nothing
                    arOrders.Sort(myComparer)
                    For Each oData In arOrders
                        'WritePOCData(1, oData)
                        If (LastData Is Nothing) Then
                            LastData = oData
                        ElseIf (oData.Medcinid = LastData.Medcinid) Then
                            LastData = oData    ' Just keep the last matching one
                        ElseIf (LastData IsNot Nothing) Then
                            ' Add this clinical order to the Clinical Orders node
                            AddPlanOfCareOrder(LastData)
                            LastData = oData
                        End If
                    Next
                    If (LastData IsNot Nothing) Then
                        AddPlanOfCareOrder(LastData)
                        LastData = Nothing
                    End If
                End If
                ' If we have nursing diagnoses, process them
                If (arDx.Count > 0) Then
                    Dim LastData As CTreeNodeTag = Nothing
                    ' Sort the list
                    arDx.Sort(myComparer)
                    For Each oData In arDx
                        'WritePOCData(1, oData)
                        If (LastData Is Nothing) Then
                            LastData = oData
                        ElseIf (oData.Medcinid = LastData.Medcinid) Then
                            LastData = oData    ' Just keep the last matching one
                        ElseIf (LastData IsNot Nothing) Then
                            AddPlanOfCareRNDx(LastData)
                            LastData = oData
                        End If
                    Next
                    If (LastData IsNot Nothing) Then
                        AddPlanOfCareRNDx(LastData)
                        LastData = Nothing
                    End If
                    ' If we have nursing interventions, process them
                    If (arRx.Count > 0) Then
                        arRx.Sort(myComparer)
                        For Each oData In arRx
                            'WritePOCData(1, oData)
                            If (LastData Is Nothing) Then
                                LastData = oData
                            ElseIf (oData.Medcinid = LastData.Medcinid) Then
                                LastData = oData    ' Just keep the last matching one
                            ElseIf (LastData IsNot Nothing) Then
                                AddPlanOfCareRNRx(LastData)
                                LastData = oData
                            End If
                        Next
                        If (LastData IsNot Nothing) Then
                            AddPlanOfCareRNRx(LastData)
                            LastData = Nothing
                        End If
                        If (arOx.Count > 0) Then
                            arOx.Sort(myComparer)
                            For Each oData In arOx
                                'WritePOCData(1, oData)
                                If (LastData Is Nothing) Then
                                    LastData = oData
                                ElseIf (oData.Medcinid = LastData.Medcinid) Then
                                    LastData = oData    ' Just keep the last matching one
                                ElseIf (LastData IsNot Nothing) Then
                                    AddPlanOfCareRNOx(LastData)
                                    LastData = oData
                                End If
                            Next
                            If (LastData IsNot Nothing) Then
                                AddPlanOfCareRNOx(LastData)
                                LastData = Nothing
                            End If
                        End If
                        'ColSet = utPlanCare.ColumnSettings.ColumnSets("rnox")
                        'For Each Column As UltraTreeNodeColumn In ColSet.Columns
                        '    Column.PerformAutoResize(Infragistics.Win.UltraWinTree.ColumnAutoSizeMode.AllNodes)
                        'Next
                    End If
                End If
                'bFileOpen = False
                'FileClose(1)
                'MsgBox("POC data saved to '" & FileName & "'")

        Catch ex As Exception
            PostError(ex)
        Finally
            ResizePlanOfCare()
            utPlanCare.EndUpdate()
            'If (bFileOpen) Then
            '    bFileOpen = False
            '    FileClose(1)
            'End If
        End Try
        utPlanCare.ExpandAll()
        objNode = Nothing
        objBNode = Nothing
        objSibling = Nothing
    End Sub

    'Private Sub WritePOCData(ByVal Filnum As Integer, ByVal oData As CTreeNodeTag)
    '    With oData
    '        WriteLine(1, .Caption, .CCC, .Etime, .Medcinid, .NodeKey, .Prefix, .RNType, .SIG, .Status, .Unit, .Value)
    '    End With
    'End Sub

    Private Sub ClearPlanOfCare()
        utPlanCare.Nodes.Clear()
        AddHeaders2PlanOfCare()
    End Sub

    Public Sub PostError(ByVal ex As Exception)
        MsgBox(ex.Message, MsgBoxStyle.Critical)
    End Sub

    Private Sub ResizePlanOfCare()
        If (utPlanCare.Nodes.Count = 0) Then Exit Sub
        Dim Column As UltraTreeNodeColumn
        Dim ColSet As UltraTreeColumnSet
        With utPlanCare
            ' The 60 is a hack for now assuming CellWrapText is True
            Dim ColSize As Size = New Size((.Width - (30 + SystemInformation.VerticalScrollBarWidth)) \ 4, 0)
            Dim HdrSize As Size = New Size(ColSize.Width, 0)
            .AllowAutoDragScrolling = True
            .NodeLevelOverrides(1).Multiline = DefaultableBoolean.True
            .NodeLevelOverrides(1).MaxLabelWidth = .Width
            ColSet = .ColumnSettings.ColumnSets("rnox")
            With ColSet
                .CellWrapText = DefaultableBoolean.False
                For Each Column In ColSet.Columns
                    With Column.LayoutInfo
                        .PreferredCellSize = ColSize
                        .PreferredLabelSize = HdrSize
                    End With
                    Column.PerformAutoResize(Infragistics.Win.UltraWinTree.ColumnAutoSizeMode.AllNodes)
                Next
            End With
            ColSet = .ColumnSettings.ColumnSets("order")
            With ColSet
                ' FIX: This needs to be based on the current setting expand vs collapse
                .CellWrapText = DefaultableBoolean.False
                For Each Column In .Columns
                    With Column.LayoutInfo
                        Select Case Column.Index
                            Case 0
                                .PreferredCellSize = ColSize
                                .PreferredLabelSize = HdrSize
                            Case Else
                                .PreferredCellSize = New Size(ColSize.Width * 3, 0)
                                .PreferredLabelSize = .PreferredCellSize
                                Column.PerformAutoResize(Infragistics.Win.UltraWinTree.ColumnAutoSizeMode.AllNodes)
                        End Select
                    End With
                Next
            End With
        End With
    End Sub

    Private Sub utPlanCare_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles utPlanCare.MouseDown
        myMouse = e
    End Sub

    Private Sub utPlanCare_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles utPlanCare.MouseUp
        ' The right mouse button does not fire AfterActivate, so handle here
        If (e.Button = Windows.Forms.MouseButtons.Right) Then
            Dim mainElement As UIElement = utPlanCare.UIElement ' Get the control's main element
            Dim oNode As UltraTreeNode = utPlanCare.GetNodeFromPoint(e.X, e.Y)
            'Debug.WriteLine("Right click at point(" & e.X & "," & e.Y & ")")
            If (Not oNode Is Nothing) Then
                If (oNode.Level = 0) Then
                    If (oNode.Index = 1) Then
                        ' User right clicked on the Active RNDx node
                        myPOCEventData = Nothing
                        utPlanCare.ActiveNode = oNode
                        oNode.Selected = True
                        myMenuActive.Show(utPlanCare, e.Location)
                        Exit Sub
                    End If
                ElseIf (oNode.Level = 1) Then
                    ' User clicked on a clinical order or an RNDx
                    Select Case oNode.Parent.Index
                        Case 1
                            ' Active RNDx
                            myPOCEventData = oNode.Tag
                            myMenuRNDx.MenuItems(4).Enabled = myPOCEventData.Prefix.StartsWith("G")
                            utPlanCare.ActiveNode = oNode
                            oNode.Selected = True
                            utPlanCare.Refresh()
                            myMenuRNDx.Show(utPlanCare, e.Location)
                            Exit Sub
                        Case Else
                            ' This was a clinical order or inactive RNDx - no menus for that
                            Exit Sub
                    End Select
                ElseIf (oNode.Level = 2) Then
                    ' This is an Intervention
                    myPOCEventData = oNode.Tag
                    utPlanCare.ActiveNode = oNode
                    oNode.Selected = True
                    utPlanCare.Refresh()
                    myMenuRNRx.Show(utPlanCare, e.Location)
                Else
                    Dim Menu As ContextMenu = Nothing
                    ' This is an RNOx node. We need to see what cell the user is on.
                    ' Get the element at that point
                    Dim element As UIElement = mainElement.ElementFromPoint(e.Location)
                    If element Is Nothing Then Return
                    ' Get the cell of the element.
                    Dim oCell As UltraTreeNodeCell = element.GetContext(GetType(UltraTreeNodeCell))
                    If Not oCell Is Nothing Then
                        'Debug.WriteLine("in cell column: " & oCell.Column.Index & " with text: " & oCell.Text)
                        myPOCEventData = oCell.Tag
                        If (myPOCEventData Is Nothing) Then Exit Sub
                        oNode.Selected = True
                        utPlanCare.ActiveNode = oNode
                        utPlanCare.ActiveCell = oCell
                        'oNode.Selected = False
                        utPlanCare.Refresh()
                        myMenuRNOx.Show(utPlanCare, e.Location)
                    End If
                End If
            ElseIf (mainElement IsNot Nothing) Then
                ' Get the element at that point
                Dim element As UIElement = mainElement.ElementFromPoint(e.Location)
                While element IsNot Nothing
                    Dim oHeader As UltraTreeNodeColumnHeaderUIElement = TryCast(element, UltraTreeNodeColumnHeaderUIElement)
                    If (oHeader IsNot Nothing) Then
                        Dim Index As Integer = oHeader.Column.Index + 2   ' Menu items are +2
                        For i As Integer = 2 To myMenuRNOx.MenuItems.Count - 1
                            myMenuRNOx.MenuItems(i).Enabled = i = Index
                        Next
                        oNode = oHeader.Node
                        If (oNode IsNot Nothing) Then
                            myPOCEventData = oHeader.Node.Tag
                        End If
                        myMenuRNOx.Show(utPlanCare, e.Location)
                        Exit Sub
                    End If
                    element = element.Parent
                End While
            End If
        End If
    End Sub

    Private Function ImportData(ByVal Filepath As String) As ArrayList
        Dim arData As New ArrayList
        Dim bFileOpen As Boolean
        Dim oData As CTreeNodeTag
        Try
            FileOpen(2, FilePath, OpenMode.Input)
            bFileOpen = True
            Do While Not EOF(2)
                oData = New CTreeNodeTag
                With oData
                    Input(2, .Caption)
                    Input(2, .CCC)
                    Input(2, .Etime)
                    Input(2, .Medcinid)
                    Input(2, .NodeKey)
                    Input(2, .Prefix)
                    Input(2, .RNType)
                    Input(2, .SIG)
                    Input(2, .Status)
                    Input(2, .Unit)
                    Input(2, .Value)
                    If (.RNType = CTreeNodeTag.RnTypes.Dx) Then
                        Dim sDx As String = String.Empty
                        Input(2, sDx)
                        If (sDx.Length > 1) Then
                            Dim sList As New ArrayList
                            Dim sDxs() As String = sDx.Split("|")
                            For Each sDx In sDxs
                                Dim iPos As Integer = sDx.IndexOf(",")
                                Dim oItem As New CIndexItem(CInt(sDx.Substring(0, iPos)), sDx.Substring(iPos + 1))
                                sList.Add(oItem)
                            Next
                            oData.RNRxList = sList
                        End If
                    End If
                End With
                arData.Add(oData)
            Loop
            bFileOpen = False
            FileClose(2)
        Catch ex As Exception
            PostError(ex)
        End Try
TheExit:
        If (bFileOpen) Then
            bFileOpen = False
            FileClose(2)
        End If
        Return arData
    End Function

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Dim DlgFile As New OpenFileDialog

        Try
            DlgFile.InitialDirectory = Application.ExecutablePath
            DlgFile.Filter = "txt files (*.txt)|*.txt"
            DlgFile.FilterIndex = 1
            If DlgFile.ShowDialog() = DialogResult.OK Then
                Dim FilePath As String = DlgFile.FileName
                BuildPlanOfCare(ImportData(FilePath))
            End If
        Catch ex As Exception
            PostError(ex)
        End Try
    End Sub
End Class
